// C Source File
// Created 16.02.2006; 18:18:18

 /********************************************************************************
 ********************************************************************************
 * Programm name:		thedb - The handy education database
 * File name:				main.c
 * Function name:		_main
 *
 * Short Description:	Load settings - start main menu - save settings
 *
 * Description:
 *	- Save the HOME screen
 *	- Show the "Loading..." window
 *	- Allocate memory for the configuration structure
 *	- Check if there's a backup file (backup00) in the main folder. If it is, ask
 *		to overwrite or to exit the programm.
 *	- Unarchive the configuration file and open it (or create a new one)
 *	- Load the configuration data. If it fails, check if it's a wrong file versoin
 *		and set the default settings
 *	- Close the "Loading..." window and show the main-menu
 *	- Save the configuration data and archive the configuration and the data file
 *	- Free everything and restore the HOME screen
 *
 ********************************************************************************
 *********************************************************************************/


#include <tigcclib.h>
#include "fmanage.h"

#include "structures.h"
#include "global.h"

// Main Function
void _main(void)
{
	FILE *thedbcnf = NULL;		//Configurations file thedbcnf.conf
	CONFSTRUCT *confstruct = NULL;

	int FileVersion = 0;
	int result = 0;
	int LoadingWindowActive = 0;

	void *buffer;


	// Allocate space for the LCD buffer
	if ((buffer = malloc(LCD_SIZE)) == NULL)
	{
		return;
	}
	//Save the screen
	memcpy(buffer, LCD_MEM, LCD_SIZE);


	clrscr();
	WINDOW *window = NULL;
	show_process_window(&window, "Loading");
	LoadingWindowActive = 1;

	if ((confstruct = malloc(sizeof(CONFSTRUCT))) == NULL)
		{
			//Free the timer and the window created with show_process_window.
			if (LoadingWindowActive == 1)
				{
					OSVFreeTimer(1);
				  WinClose(window);
				  HeapFreePtr(window);
				  FontSetSys(F_4x6);
				  LoadingWindowActive = 0;
				}

			DlgMessage("thedb[1]", "Memory!", BT_OK, BT_CANCEL);
			goto end;
		}

	//Check if there's a backup file in the main folder. If it is, ask if it should be removed or not.
	HSym hs = checkCurrent(SYMSTR(BACKUPFILE_NAME), OTH_TAG);
	if (hs.folder != 0)
		{
			//Free the timer and the window created with show_process_window.
			if (LoadingWindowActive == 1)
				{
					OSVFreeTimer(1);
				  WinClose(window);
				  HeapFreePtr(window);
				  FontSetSys(F_4x6);
				  LoadingWindowActive = 0;
				}
			if ((DlgMessage("thedb[44]", "There's a backup file in the main folder. If you continue, it will be removed.\n"
																		"Continue and remove backup file?", BT_YES, BT_NO)) == KEY_ENTER)
				{
					EM_moveSymFromExtMem(SYMSTR(BACKUPFILE_NAME), HS_NULL);
					unlink(BACKUPFILE_NAME);
				}
			else
				{
					goto end;
				}
		}

	//Make shure, the configuration file is not archived
	EM_moveSymFromExtMem(SYMSTR(CONFIGURATION_FILE), HS_NULL);
	//Open the configuration file or create a new one, if it does not exist.
	//If a new file has been created, fmanage returns 1.
	result = 	fmanage(&thedbcnf, CONFIGURATION_FILE, FMF_CONF_FILE_VERSION, CONF_FILE_EXTENSION, NULL, sizeof(CONFSTRUCT), 0, FMF_NEW_FILE, FMF_NEW_NO_OVERWR);


	//Try to load the configuration data.
	if ((fmanage(&thedbcnf, CONFIGURATION_FILE, FMF_CONF_FILE_VERSION, CONF_FILE_EXTENSION, confstruct, sizeof(CONFSTRUCT), 1, FMF_LOAD, 0)) < 0)
		{
			char str[200] = {0};

			//If fmanage with FMF_NEW_FILE has not returned 1 (no new file created)
			//and the file version is valid
			if ((result != 1) && (FileVersion = fmanage(&thedbcnf, NULL, 0, NULL, NULL, 0, 0, FMF_RETURN_FINFO, FMF_RETURN_FV)) > 0)
				{
					sprintf(str, "Failed to load the configuration data!\n"
												"Your configuration file version %d.\n"
												"This version of thdb uses version %d.\n"
												"Set to default?", FileVersion, FMF_CONF_FILE_VERSION);

					//Free the timer and the window created with show_process_window.
					if (LoadingWindowActive == 1)
						{
							OSVFreeTimer(1);
						  WinClose(window);
						  HeapFreePtr(window);
						  FontSetSys(F_4x6);
						  LoadingWindowActive = 0;
						}

					if ((DlgMessage("thedb[2]", str, BT_YES, BT_NO)) == KEY_ESC)
						goto end;
				}

			//Open the configuration file or create a new one, if it does not exist
			fmanage(&thedbcnf, CONFIGURATION_FILE, FMF_CONF_FILE_VERSION, CONF_FILE_EXTENSION, NULL, sizeof(CONFSTRUCT), 0, FMF_NEW_FILE, FMF_NEW_OVERWR);

			//If it fails, set the default settings
			strcpy(confstruct->PathFormula[0], STR_NO_FILE_SEL);
			strcpy(confstruct->PathFormula[1], STR_NO_FILE_SEL);
			strcpy(confstruct->PathFormula[2], STR_NO_FILE_SEL);
			confstruct->ActivePathFormula = 0;
			strcpy(confstruct->PathText[0], STR_NO_FILE_SEL);
			strcpy(confstruct->PathText[1], STR_NO_FILE_SEL);
			strcpy(confstruct->PathText[2], STR_NO_FILE_SEL);
			confstruct->ActivePathText = 0;

			confstruct->ActiveForm = FORM_FORMULA;
			confstruct->ActiveList = NAME;
			confstruct->PosFormula[0] = 1;
			confstruct->PosFormula[1] = 1;
			confstruct->PosFormula[2] = 1;
			confstruct->PosText[0] = 1;
			confstruct->PosText[1] = 1;
			confstruct->PosText[2] = 1;
			confstruct->StartFormula[0] = 1;
			confstruct->StartFormula[1] = 1;
			confstruct->StartFormula[2] = 1;
			confstruct->StartText[0] = 1;
			confstruct->StartText[1] = 1;
			confstruct->StartText[2] = 1;
			confstruct->Startup	= 0;
			confstruct->ArchiveFiles = 1;
			confstruct->MakeBackupCopy = 1;

			strcpy(confstruct->LastSearchString, "");

			confstruct->exact_approx = 1;
			confstruct->FontLargeSmall = 1;

			//Add the configuration structure to the configuration file
			fmanage(&thedbcnf, CONFIGURATION_FILE, FMF_CONF_FILE_VERSION, CONF_FILE_EXTENSION, confstruct, sizeof(CONFSTRUCT), 0, FMF_ADD, 0);

			if (!(result != 1) && !(FileVersion > 0))
				{
					//Free the timer and the window created with show_process_window.
					if (LoadingWindowActive == 1)
						{
							OSVFreeTimer(1);
						  WinClose(window);
						  HeapFreePtr(window);
						  FontSetSys(F_4x6);
						  LoadingWindowActive = 0;
						}
				}


		  //------------------------------
		  //Show the main menu and set cursor positions to default
				main_menu(confstruct, 1);
			//------------------------------
		}
	else
		{
			//Free the timer and the window created with show_process_window.
			if (LoadingWindowActive == 1)
				{
					OSVFreeTimer(1);
				  WinClose(window);
				  HeapFreePtr(window);
				  FontSetSys(F_4x6);
				  LoadingWindowActive = 0;
				}

		  //------------------------------
		  //Show the main menu
				main_menu(confstruct, 0);
			//------------------------------
		}


	//Update the configuration structure.
	fmanage(&thedbcnf, CONFIGURATION_FILE, FMF_CONF_FILE_VERSION, CONF_FILE_EXTENSION, confstruct, sizeof(CONFSTRUCT), 1, FMF_EDIT, FMF_CLOSE);


	archive_active_file(confstruct);
	//Archive the configuration file
	if (confstruct->ArchiveFiles == S_ARCHIVE_YES)
		EM_moveSymToExtMem(SYMSTR(CONFIGURATION_FILE), HS_NULL);

end:
	free(confstruct);
	//restore the screen
	memcpy(LCD_MEM, buffer, LCD_SIZE);
	free(buffer);
	GrayOff();

	ST_helpMsg("thedb - by Thomas Naeff");

	return;

}
