// C Source File
// Created 18.02.2006; 09:06:47

 /********************************************************************************
 ********************************************************************************
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_control
 * Header file:			main_menu.h
 * Other functions:
 *	- main_menu_set_form
 *	- main_menu_cursor
 *	- main_menu_switch_gk
 *	- main_menu_process_enter
 *	- main_menu_show_info
 *	- main_menu_info_cursor
 *	- main_menu_info_switch_gk
 *	- main_menu_info_process_enter
 *	- main_menu_info_process_clear
 *	- main_menu_info_process_backspace
 *	- main_menu_info_process_2nd_right_left
 *	- main_menu_info_process_mode
 *	- main_menu_switch_menu0
 *
 * Short Description:	Show the cursor on it's position - check the keypress - process
 											the keypress - remove the old cursor - restart
 *
 * Description:
 *	- Show the cursor in the main or in the info section
 *	- Wait for a keypress
 *	- Exit if ESC was pressed
 *	- Remove the cursor
 *	- Process the keypress, depending if the cursor was in the main or in the info
 *		section
 *
 ********************************************************************************
 *********************************************************************************/

#include <tigcclib.h>
#include "fmanage.h"

#include "structures.h"
#include "global.h"
#include "calc.h"
#include "main_menu.h"


//Process all keypresses in the whole main menu
int main_menu_control (CONFSTRUCT *confstruct, int *ActiveMenu, int *CursorPosY)
{

	int gk = 0;
	int result = 0;

	GraySetAMSPlane(DARK_PLANE);


		while (1)
			{
				//Show the cursor
				if (*ActiveMenu == MM_MAIN)
					main_menu_cursor(*CursorPosY);
				else
					main_menu_info_cursor(*CursorPosY);

				//Wait for a keypress
				gk = ngetchx();

				//Exit the Programm, if ESC was pressed
				if (gk == KEY_ESC)
					{
						return -1;
					}


				if (*ActiveMenu == MM_MAIN)
					{
						//Remove the old cursor
						main_menu_cursor(*CursorPosY);

						result = main_menu_switch_gk(confstruct, gk, CursorPosY, ActiveMenu);
						if (result == -1)
							return -1;
						if (result == 1)
							return 1;
					}
				else
					{
						//Remove the old cursor
						main_menu_info_cursor(*CursorPosY);

						result = main_menu_info_switch_gk(confstruct, gk, CursorPosY, ActiveMenu);
						if (result == -1)
							return -1;
						if (result == 1)
							return 1;
					}

				ST_helpMsg("Use UP/DOWN or RIGHT/LEFT to navigate");
			};


	return -1;
}


 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_set_form
 *
 * Short Description:	Show whether "Text" or "Formula" is choosen
 *
 * Description:
 *	- Remove the old text (Overwrite it with spaces)
 *	- Copy the new text into a buffer
 *	- Write the new text on the GRAY and LIGHT plane
 *
 *********************************************************************************/
//Set the active form (Formula/Text/...)
void main_menu_set_form (CONFSTRUCT *confstruct)
{
	char str[20] = {0};

	//Clear the old text
	GraySetAMSPlane (LIGHT_PLANE);
	DrawStr(8, MM_MENU0, "                 ", A_REPLACE);
	GraySetAMSPlane (DARK_PLANE);
	DrawStr(8, MM_MENU0, "                 ", A_REPLACE);

	switch (confstruct->ActiveForm)
		{
			case FORM_TEXT:
					strcpy(str, "Text");
				break;
			case FORM_FORMULA:
			default:
					confstruct->ActiveForm = FORM_FORMULA;
					strcpy(str, "Formula");
				break;
				break;
		}

	//Set the new one
	DrawStr(8, MM_MENU0, str, A_NORMAL);
	GraySetAMSPlane (LIGHT_PLANE);
	DrawStr(8, MM_MENU0, str, A_NORMAL);



	return;
}


 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_cursor
 *
 * Short Description:	Show the cursor in the main section (The line and the triangle)
 *
 * Description:
 *	- Set the DARK plane
 *	- Draw the triangles on the left and on the right
 *	- Draw a pixel on the left
 *	- Choose the lenght of the line
 *	- Draw the line on the right side
 *
 *********************************************************************************/
//Show the cursor in the main menu
void main_menu_cursor (int CursorPosY)
{
	int lenght = 1;

	GraySetAMSPlane (DARK_PLANE);

	//Show/remove the "cursor"
	DrawChar(2, CursorPosY, 0x12, A_XOR);
	DrawChar(52, CursorPosY, 0x11, A_XOR);
	DrawPix(7, CursorPosY + 2, A_XOR);

	switch (CursorPosY)
		{
			case MM_MENU0:
					lenght = 7;
				break;
			case MM_MENU1:
					lenght = 24;
				break;
			case MM_MENU2:
					lenght = 27;
				break;
			case MM_MENU3:
					lenght = 15;
				break;
			case MM_MENU4:
					lenght = 17;
				break;
			case MM_MENU5:
					lenght = 29;
				break;
			default:
				break;
		}

	DrawLine(52 - lenght, CursorPosY + 2, 51, CursorPosY + 2, A_XOR);

	return;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_switch_gk
 *
 * Short Description: Check the pressed key and execute the function
 *
 * Description:
 *	- KEY_DOWN: move the cursor downwards. If it reached the bottom, set to top
 *	- KEY_UP: move the cursor upwards. If it reached the top, set to bottom
 *	- KEY_ENTER: Start the function to choose what to do
 *	- KEY_RIGHT: If the cursor is not in the Formula/Text-switch-section, move it
 *								to the info section
 *	- KEY_LEFT: Switch the Formula/Text-section
 *	- KEY_ON: Switch the calculator off
 *
 *********************************************************************************/
//Process the keypresses in the main menu
int main_menu_switch_gk (CONFSTRUCT *confstruct, int gk, int *CursorPosY, int *ActiveMenu)
{

				switch(gk)
					{
						case KEY_DOWN:
								if ((*CursorPosY + 10) > MM_MENU5)
									{
										*CursorPosY = MM_MENU0;
										break;
									}

								*CursorPosY += 10;
							break;
						case KEY_UP:
								if ((*CursorPosY - 10) < MM_MENU0)
									{
										*CursorPosY = MM_MENU5;
										break;
									}

								*CursorPosY -= 10;
							break;
						case KEY_ENTER:
								if ((main_menu_process_enter(confstruct, *CursorPosY, *ActiveMenu)) == -1)
									return -1;

								return 1;
							break;
						case KEY_RIGHT:
								if (*CursorPosY == MM_MENU0)
									main_menu_switch_menu0(confstruct, gk);

								if (*CursorPosY != MM_MENU0)
									{
										*CursorPosY = MM_INFO_ACTIVE_FILE;
										*ActiveMenu = MM_INFO;
									}
							break;
						case KEY_LEFT:
							if (*CursorPosY == MM_MENU0)
								main_menu_switch_menu0(confstruct, gk);
							break;
						case KEY_ON:
								off();
							break;
						default:
							break;
					}

	return 0;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_process_enter
 *
 * Short Description: Check what to do if enter has been pressed in the main section
 *
 * Description:
 *	- Check if the active form is set to Formula or to Text
 *	- Execute the corresponding function, depending on the menu position where the
 *		cursor stands
 *	- Formula or Text: Show the INFOMESSAGE or exit the programm
 *
 *********************************************************************************/
//Process the enter keypress from the main menu
int main_menu_process_enter (CONFSTRUCT *confstruct, int CursorPosY, int ActiveMenu)
{

#ifdef THDB_AUTOSAVE
	int x = 0;
#endif

	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
						switch(CursorPosY)
							{
								case MM_MENU0:
									break;
								case MM_MENU1:
										while ((list_data(confstruct)) == 1);
									break;
								case MM_MENU2:
									#ifdef THDB_AUTOSAVE
										for (x = 0; x < THDB_AUTOSAVE_NUMBER; x++)
									#endif
											new_entry_formula(confstruct, FMF_ADD, 0);
									break;
								case MM_MENU3:
									break;
								default:
									break;
							}
				break;
			case FORM_TEXT:
						switch(CursorPosY)
							{
								case MM_MENU0:
									break;
								case MM_MENU1:
										while ((list_data(confstruct)) == 1);
									break;
								case MM_MENU2:
									#ifdef THDB_AUTOSAVE
										for (x = 0; x < THDB_AUTOSAVE_NUMBER; x++)
									#endif
										new_entry_text(confstruct, FMF_ADD, 0);
									break;
								case MM_MENU3:
									break;
								default:
									break;
							}
				break;
			default:
				break;
		}


	switch(CursorPosY)
		{
			case MM_MENU3:
					CursorPosY = MM_SETTINGS_STARTUP;
					ActiveMenu = MM_SETTINGS;
					while ((settings(confstruct, &CursorPosY)) != -1);
				break;
			case MM_MENU4:
					GrayOff();
					clrscr();
					INFOMESSAGE;
				break;
			case MM_MENU5:
					return -1;
				break;
			default:
				break;
		}


	return 0;
}


 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_show_info
 *
 * Short Description: Show the informations at the right side of the main menu
 *
 * Description:
 *	- Draw everything on the LIGHT plane
 *	- Clear the old "Active file" number (overwrite it with spaces) and set the new one
 *	- Clear the old text of the file path (overwrite it with spaces)
 *	- Set the active path (saved in the variable ActivePath -> global.h)
 *	- Get the file extension of the active file and print out the active path with
 *		the file extension
 *	- Set the simbol which shows if the file is locked/not locked or somethin was wrong
 *		to default
 *	- Check if the file-in-use bit of the file is set
 *	- Try to open the active file. If it's ok...
 *	- ...make a backup copy of it
 *	- ...unarchive the file
 *	- ...open the file and load the number of stored blocks
 *	- ...Set the simbol which shows if the file is locked/not locked or somethin was wrong
 *	- Print out the number of blocks in the file in the middle of the footer of the
 *		main menu
 *
 *********************************************************************************/
//Show the infos on the right display side
void main_menu_show_info (CONFSTRUCT *confstruct, int silent)
{
	FILE *datafile = NULL;		//The file pointer that handles the data file
	char str[50] = {0};
	char ext[5] = {0};
	long int NumOfBlocks = 0;

	GraySetAMSPlane (LIGHT_PLANE);

	//-------------------------------------------------------------------
	//Clear the old number
	if (silent == 0)
		DrawStr(100, MM_INFO_ACTIVE_FILE - 8, "     ", A_REPLACE);
	//Set the number of the active file path
	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
					sprintf(str, "[%d]", confstruct->ActivePathFormula + 1);
				break;
			case FORM_TEXT:
					sprintf(str, "[%d]", confstruct->ActivePathText + 1);
				break;
			default:
				break;
		}
	//Draw the number
	if (silent == 0)
		DrawStr(100, MM_INFO_ACTIVE_FILE - 8, str, A_NORMAL);

	//-------------------------------------------------------------------
	//Clear the old text
	if (silent == 0)
		{
			DrawStr(70, MM_INFO_ACTIVE_FILE, "                                      ", A_REPLACE);
			DrawStr(70, LCD_HEIGHT - 15, "       ", A_REPLACE);
		}


	//Set the file name and path
	switch (confstruct->ActiveForm)
		{
			case FORM_FORMULA:
					strcpy(ActivePath, confstruct->PathFormula[confstruct->ActivePathFormula]);
				break;
			case FORM_TEXT:
					strcpy(ActivePath, confstruct->PathText[confstruct->ActivePathText]);
				break;
			default:
				break;
		}

	//Get file extension
	FType(ActivePath, ext);
	//Print out the active file path
	sprintf(str, "%s.%s", ActivePath, ext);
	if (silent == 0)
		DrawStr(70, MM_INFO_ACTIVE_FILE, str, A_NORMAL);
	//Print out, if autostart is set
	//DrawStr(70, MM_INFO_AUTOSTART, " Information", A_NORMAL);

	//Set the old locked/not locked symbol to default
	if (silent == 0)
		{
			DrawStr(110, MM_INFO_ACTIVE_FILE - 8, "   ", A_REPLACE);
			DrawChar(110, MM_INFO_ACTIVE_FILE - 8, 0x3F, A_REPLACE);
		}


	if (check_if_in_use_bit_set(1) == -1)
		return;

	sprintf(str, "---");
	//Try to open the active file if there's a real path in it
	if (strstr(ActivePath, STR_NO_FILE_SEL) == 0)
		{
			//Try to make a backup copy. There is no check if it was successfull.-----------------------------------
			if (confstruct->MakeBackupCopy == S_BACKUP_YES)
				{
					ST_helpMsg("Make backup copy...");
					TRY
						cmd_copyvar(SYMSTR(ActivePath), SYMSTR(BACKUPFILE_NAME));
						EM_moveSymToExtMem(SYMSTR(BACKUPFILE_NAME), HS_NULL);
					ONERR
					ENDTRY
				}

			//Make shure the file is unarchived
			ST_helpMsg("Unarchieve new file...");
			EM_moveSymFromExtMem(SYMSTR(ActivePath), HS_NULL);
			//Open the file
			if ((datafile = fopen(ActivePath, "rb")) != NULL)
				{
					//Load the number of blocks
					NumOfBlocks = fmanage(&datafile, NULL, 0, NULL, NULL, 0, 0, FMF_RETURN_FINFO, FMF_RETURN_NOB);
					sprintf(str, "%3ld", NumOfBlocks);

					//Check if locked or not
					//Clear the old locked/not locked symbol
					if (silent == 0)
						DrawStr(110, MM_INFO_ACTIVE_FILE - 8, "   ", A_REPLACE);
					//Show the symbols for locked or no locked
					if ((fmanage(&datafile, NULL, 0, NULL, NULL, 0, 0, FMF_RETURN_FINFO, FMF_RETURN_L)) == FMF_LOCK_ON)
						{
							if (silent == 0)
								DrawChar(110, MM_INFO_ACTIVE_FILE - 8, 0xE, A_NORMAL);	//Locked
						}
					else
						{
							if (silent == 0)
								DrawChar(110, MM_INFO_ACTIVE_FILE - 8, 0xF, A_NORMAL);	//Not locked
						}

					fclose(datafile);
				}

			ST_eraseHelp();
		}
	//Print out the number of blocks in the file
	if (silent == 0)
		DrawStr(70, LCD_HEIGHT - 15, str, A_XOR);
	//-------------------------------------------------------------------

	return;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_info_cursor
 *
 * Short Description:	Show the cursor in the info section (The line and the triangle)
 *
 * Description:
 *	- Draw the triangles on the left and on the right
 *	- Draw a pixel on the left
 *	- Choose the lenght of the line
 *	- Draw the line on the right side
 *
 *********************************************************************************/
//Show the cursor on the right display side
void main_menu_info_cursor (int CursorPosY)
{
	int lenght = 1;

	//Show/remove the "cursor"
	DrawChar(64, CursorPosY, 0x12, A_XOR);
	DrawChar(LCD_WIDTH - 8, CursorPosY, 0x11, A_XOR);
	DrawPix(69, CursorPosY + 2, A_XOR);

	switch (CursorPosY)
		{
			case MM_INFO_ACTIVE_FILE:
					lenght = 5;
				break;
			//case MM_INFO_AUTOSTART:
			//		lenght = 24;
				break;
			default:
				break;
		}

	DrawLine(LCD_WIDTH - 8 - lenght, CursorPosY + 2, LCD_WIDTH - 9, CursorPosY + 2, A_XOR);


	return;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_info_switch_gk
 *
 * Short Description: Check the pressed key and execute the function
 *
 * Description:
 *	- KEY_DOWN: move the cursor downwards. If it reached the bottom, set to top
 *	- KEY_UP: move the cursor upwards. If it reached the top, set to bottom
 *	- KEY_ENTER: Start the function to let the user select a file
 *	- KEY_CLEAR: Start the function to clear the active file
 *	- KEY_BACKSPACE: Start the function to create a new file
 *	- KEY_2ND+KEY_RIGHT/LEFT: Start the function to switch between the three
 *														selected files
 *	- KEY_MODE: Lock or unlock the active file
 *	- KEY_ON: switch the calculator off
 *
 *********************************************************************************/
//Process the keys pressed by the user, when the cursor is on the right display side
int main_menu_info_switch_gk (CONFSTRUCT *confstruct, int gk, int *CursorPosY, int *ActiveMenu)
{

				switch(gk)
					{
						case KEY_DOWN:
								if ((*CursorPosY + 20) > MM_INFO_AUTOSTART)
									{
										*CursorPosY = MM_INFO_ACTIVE_FILE;
										break;
									}

								*CursorPosY += 20;
							break;
						case KEY_UP:
								if ((*CursorPosY - 20) < MM_INFO_ACTIVE_FILE)
									{
										*CursorPosY = MM_INFO_AUTOSTART;
										break;
									}

								*CursorPosY -= 20;
							break;
						case KEY_LEFT:
								*CursorPosY = MM_MENU0;
								*ActiveMenu = MM_MAIN;
							break;
						case KEY_ENTER:
									if ((main_menu_info_process_enter(confstruct, *CursorPosY)) == 1)
										return 1;
							break;
						case KEY_CLEAR:
									if ((main_menu_info_process_clear(confstruct, *CursorPosY)) == 1)
										return 1;
							break;
						case KEY_BACKSPACE:
								if ((main_menu_info_process_backspace(confstruct, *CursorPosY)) == -1)
									return -1;

								return 1;
							break;
						case KEY_2ND + KEY_RIGHT:
						case KEY_2ND + KEY_LEFT:
								main_menu_info_process_2nd_right_left(confstruct, gk, *CursorPosY);
								return 1;
							break;
						case KEY_MODE:
								main_menu_info_process_mode(confstruct, *CursorPosY);
								return 1;
							break;
						case KEY_ON:
								off();
							break;
						default:
							break;
					}

	return 0;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_info_process_backspace
 *
 * Short Description: Show the dialog to create a new file
 *
 * Description:
 *	- Check the position of the cursor in the info section and execute the
 *		corresponding function
 *
 *********************************************************************************/
//Show the dialog to create a new file
int main_menu_info_process_backspace (CONFSTRUCT *confstruct, int CursorPosY)
{


	switch (CursorPosY)
		{
			case MM_INFO_ACTIVE_FILE:
					create_new_file(confstruct);
				break;
			//case MM_INFO_AUTOSTART:
			//	break;
			default:
				break;
		}

	return 0;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_info_process_clear
 *
 * Short Description: Show the dialog to delete the active file
 *
 * Description:
 *	- Check the cursor position
 *	- Check if the "Active file" path is set to STR_NO_FILE_SEL
 *	- Open the active file and check if the file is locked
 *	- Show an info dialog box and ask if the file should be removed
 *	- Remove the file and set the file path to default (STR_NO_FILE_SEL)
 *
 *********************************************************************************/
//Delete the file of the active file path
int main_menu_info_process_clear (CONFSTRUCT *confstruct, int CursorPosY)
{
	FILE *datafile = NULL;

		if (CursorPosY == MM_INFO_ACTIVE_FILE)
			{
				if (strstr(ActivePath, STR_NO_FILE_SEL) != NULL)
					return 0;

				if ((datafile = fopen(ActivePath, "rb")) != NULL)
					{
						if ((check_if_file_is_locked(datafile)) == -1)
							{
								fclose(datafile);
								return 1;
							}

						fclose(datafile);
					}


				GrayOff();
				clrscr();
				if ((DlgMessage("Remove", "Do you want to delete this file?\n"
																	"The file will be completely removed from the calculator!", BT_OK, BT_CANCEL)) == KEY_ENTER)
					{
						unlink(ActivePath);
						//Set the file path to default
						switch (confstruct->ActiveForm)
							{
								case FORM_FORMULA:
										strcpy (confstruct->PathFormula[confstruct->ActivePathFormula], STR_NO_FILE_SEL);
									break;
								case FORM_TEXT:
										strcpy (confstruct->PathText[confstruct->ActivePathText], STR_NO_FILE_SEL);
									break;
								default:
									break;
							}

						//Remove the backup copy
						if (confstruct->MakeBackupCopy == S_BACKUP_YES)
							{
								ST_helpMsg("Remove backup copy...");
								EM_moveSymFromExtMem(SYMSTR(BACKUPFILE_NAME), HS_NULL);
								if (unlink(BACKUPFILE_NAME) == EOF)
									{
										ST_helpMsg("Remove backup copy... FAILED!!!");
										ngetchx();
									}
							}

					}

				return 1;
			}

		return 0;
}


 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_info_process_enter
 *
 * Short Description: Show the var-link dialog and let the user select a file
 *
 * Description:
 *	-
 *
 *********************************************************************************/
//Show the var link dialog and let the user select a file
//Check the selected file for validity
int main_menu_info_process_enter (CONFSTRUCT *confstruct, int CursorPosY)
{
	char folder[9] = {0};
	char tempPath[PATH_LENGHT] = {0};

		if (CursorPosY == MM_INFO_ACTIVE_FILE)
			{
				GrayOff();
				clrscr();
				memset(VarBuffer, 0, sizeof(VarBuffer));
				ShowVarLink();
				if (strlen(VarBuffer) < 1)
					return 1;

				//Save the active path.
				//If the selected file is not valid, it will be copied back later
				strcpy(tempPath, ActivePath);

				//Archive the old file
				archive_active_file(confstruct);

				//Check if a \ is in the pathname.
				//If \ is not found, get the active folder and add it in front of the filename to the pathname
				if ((strstr(VarBuffer, "\\")) == NULL)
					{
						//Get the active folder
						FolderGetCur(folder);
						//Set the path name
						sprintf(ActivePath, "%s\\%s", folder, VarBuffer);
					}
				else
					{
						//Copy the pathname
						strcpy(ActivePath, VarBuffer);
					}

				clrscr();
				//Check if the file path is valid
				if ((check_file_validity(confstruct, ActivePath, 1)) == 0)
					{
						//If the selected file is not valid, copy back
						strcpy(ActivePath, tempPath);
					}
				else
					{
						//Copy the pathname
						switch (confstruct->ActiveForm)
							{
								case FORM_FORMULA:
										strcpy(confstruct->PathFormula[confstruct->ActivePathFormula], ActivePath);
									break;
								case FORM_TEXT:
										strcpy(confstruct->PathText[confstruct->ActivePathText], ActivePath);
									break;
								default:
									break;
							}
					}

				return 1;

			}

	/*
		if (CursorPosY == MM_INFO_AUTOSTART)
			{

			}
	*/

	return 0;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_info_process_2nd_right_left
 *
 * Short Description: Switch the "Active file" path (1-3)
 *
 * Description:
 *	-
 *
 *********************************************************************************/
//Loop through the three stored file path's
void main_menu_info_process_2nd_right_left (CONFSTRUCT *confstruct, int gk, int CursorPosY)
{

	if (CursorPosY == MM_INFO_ACTIVE_FILE)
		{
				//First, archive the active file, and then switch to the new one
				archive_active_file(confstruct);
				switch(gk)
					{
						case KEY_2ND + KEY_RIGHT:
								switch (confstruct->ActiveForm)
									{
										case FORM_FORMULA:
												if ((confstruct->ActivePathFormula + 1) > 2)
													{
														confstruct->ActivePathFormula = 0;
														break;
													}

												confstruct->ActivePathFormula = confstruct->ActivePathFormula + 1;
											break;
										case FORM_TEXT:
												if ((confstruct->ActivePathText + 1) > 2)
													{
														confstruct->ActivePathText = 0;
														break;
													}
												confstruct->ActivePathText = confstruct->ActivePathText + 1;
											break;
										default:
											break;
									}
							break;
						case KEY_2ND + KEY_LEFT:
								switch (confstruct->ActiveForm)
									{
										case FORM_FORMULA:
												if ((confstruct->ActivePathFormula - 1) < 0)
													{
														confstruct->ActivePathFormula = 2;
														break;
													}
												confstruct->ActivePathFormula = confstruct->ActivePathFormula - 1;
											break;
										case FORM_TEXT:
												if ((confstruct->ActivePathText - 1) < 0)
													{
														confstruct->ActivePathText = 2;
														break;
													}
												confstruct->ActivePathText = confstruct->ActivePathText - 1;
											break;
										default:
											break;
									}
							break;
						default:
							break;
					}

		}

	return;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_info_process_mode
 *
 * Short Description: Lock or unlock the active file
 *
 * Description:
 *	-
 *
 *********************************************************************************/
//Lock or unlock the file
void main_menu_info_process_mode (CONFSTRUCT *confstruct, int CursorPosY)
{
	FILE *f = NULL;
	char extension[5] = {0};

	if (CursorPosY == MM_INFO_ACTIVE_FILE)
		{

			switch (confstruct->ActiveForm)
				{
					case FORM_FORMULA:
							strcpy(extension, FILE_FORMULA_EXTENSION);
						break;
					case FORM_TEXT:
							strcpy(extension, FILE_TEXT_EXTENSION);
						break;
					default:
						break;
				}

	GrayOff();
	clrscr();

	if ((check_file_validity(confstruct, ActivePath, 1)) == 0)
		return;

			if ((f = fopen(ActivePath, "rb+")) != NULL)
				{
					WINDOW *window = NULL;
					show_process_window (&window, "Lock/Unlock");
					//Check if locked or not
					//Swap lock
					if ((fmanage(&f, NULL, 0, NULL, NULL, 0, 0, FMF_RETURN_FINFO, FMF_RETURN_L)) == FMF_LOCK_ON)
						fmanage(&f, ActivePath, 0, extension, NULL, 0, 0, FMF_EDIT, FMF_UNLOCK_FILE | FMF_NO_FINFO_CHECK);
					else
						fmanage(&f, ActivePath, 0, extension, NULL, 0, 0, FMF_EDIT, FMF_LOCK_FILE | FMF_NO_FINFO_CHECK);

					//Free the timer and the window created with show_process_window.
					OSVFreeTimer (1);
				  WinClose (window);
				  HeapFreePtr (window);

					fclose(f);
				}

		}


	return;
}

 /********************************************************************************
 * - Subfunction of main_menu_control -
 *
 * Programm name:		thedb - The handy education database
 * File name:				main_menu_control.c
 * Function name:		main_menu_switch_menu0
 *
 * Short Description: Switch the active form (Formula/Text), depending on the keypress
 *
 * Description:
 *	-
 *
 *********************************************************************************/

void main_menu_switch_menu0 (CONFSTRUCT *confstruct, int gk)
{
	archive_active_file(confstruct);

				switch(gk)
					{
						case KEY_RIGHT:
								if ((confstruct->ActiveForm + 1) > FORM_TEXT)
									{
										confstruct->ActiveForm = FORM_FORMULA;
										break;
									}

								confstruct->ActiveForm = confstruct->ActiveForm + 1;
							break;
						case KEY_LEFT:
								if ((confstruct->ActiveForm - 1) < FORM_FORMULA)
									{
										confstruct->ActiveForm = FORM_TEXT;
										break;
									}

								confstruct->ActiveForm = confstruct->ActiveForm - 1;
							break;
						default:
							break;
					}

	main_menu_set_form(confstruct);
	main_menu_show_info(confstruct, 0);


	return;
}


//==================================================================================================

//==================================================================================================




