// C Source File
// Created 11.03.2006; 14:38:55

 /********************************************************************************
 ********************************************************************************
 * Programm name:		thedb - The handy education database
 * File name:				read_vars_of_expr.c
 * Function name:		read_vars_of_expr
 * Header file:			---
 * Other functions:
 *	-
 *
 * Short Description: read the variable names of an expression.
 *										I've downloaded the original code of getnames.zip at:
 *										http://www.ticalc.org/archives/files/fileinfo/274/27424.html
 *
 * Description:
 *	-
 *
 ********************************************************************************
 *********************************************************************************/


/**********************************************
 *By Samuel Stearley copyright 2002
 *
 *http://www.geocities.com/sstear70/
 *
 *
 *If you examine the code carefully you will
 *  realize that in the course of the loop ptr
 *  will point to every tag on the estack.
 **********************************************
 *
 * Modified by: Thomas Naeff
 * -> marked with start: //>>>>>
 *								end:   //<<<<<
 *
 **********************************************/

#include <tigcclib.h>
#include "structures.h"

int read_vars_of_expr (const char *expr, char ListOfVars[MAXLEN_VARS])
{
	//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	//Save stack pointer
	unsigned char * temp_top_estack = top_estack;

	memset(ListOfVars, 0, sizeof(ListOfVars));
	reset_control_flags();

	push_END_TAG();
	TRY
		push_parse_text(expr);
	ONERR
			DlgMessage("thedb[32]", "Can't solve this expression!", BT_OK, BT_NONE);
			//Restore stack pointer
			top_estack = temp_top_estack;
			return -1;
	ENDTRY
	//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

	unsigned char * ptr = top_estack;
	unsigned char * source = ptr;
	push_END_TAG();
	for (;;){
		unsigned char d = *ptr;
		if (d < 0x2E){			//is it a var, float, fraction, ect?
			unsigned char * temp = ptr;
			ptr = next_expression_index(ptr);
			if (d < 0x1C){		//is it a var name?
				if (is_units_term(temp) > 0) continue;
				if (is_tail_independent_of(top_estack,temp)) // prevent duplicates
					//printf("%d: ptr: %s\n", d, ptr);
					push_between(ptr,temp);	//push it to my list


			}
		}
		else{
			--ptr;
			if (d == USERFUNC_TAG) ptr=next_expression_index(ptr);
/*************************************************************************
 *Sorry for the next lines, the following would be more clear:
 *
 * if ((d == (unsigned char)EXT_TAG) || (d == (unsigned char)EXT_INSTR_TAG))
 *		--ptr;
 *
 *But in the interest of keeping the asm comparison fair I am making an
 *optomization that the compiler was making itself so that other
 *optomizations could be done.
 ************************************************************************/
			d -= (unsigned char)EXT_TAG; //d=0 if ext tag or 1 if basic tag
			if (d <= 1)					// check if it is either of the extended tags, this
				--ptr;
/**************************************************
 *Again sorry for the lack of clarity  This will
 *  check if we have hit the end_estack_tag
 *
 *I am curious if tigcc will optomize this with a
 *  subq.b #6,dx or if it will use a compare?
 *
 *I checked and it uses a compare so I changed the
 *  code to "if ((d-6) == 0) break;" but it still
 *  used a compare.
 *
 *I also tried if ((d -=6) == 0) break; but it still
 *  used a compare.
 *
 * a subq.b #6,dx is 2 bytes and 4 clocks
 * a cmp.b #6,dx is 4 bytes and 8 clocks
 *************************************************/

			if (d ==6) break;
		}
	};
	push_LIST_TAG();
	char * temp = top_estack;
	top_estack = ptr+1;
	push_between(source,temp);  //copy it to the bottom of the estack

	//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	//--- Get the list of variables from the expression stack and return them ---
	HANDLE handle = H_NULL;

	handle = display_statements(top_estack, 1, 1);

	//Copy the list of variables into ListOfVars
	strncpy (ListOfVars, HeapDeref(handle) + 1, MAXLEN_VARS);	//Cut the first {
	ListOfVars[strlen(ListOfVars) - 1] = '\0';										//and the las } off
	HeapFree(handle);

	//Restore stack pointer
	top_estack = temp_top_estack;
	//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

	return 0;
}

