// C Source File
// Created 06.03.2006; 16:14:51

 /********************************************************************************
 ********************************************************************************
 * Programm name:		thedb - The handy education database
 * File name:				view2d.c
 * Function name:		view2d
 * Header file:			---
 * Other functions:
 *	- win_reopen
 *	- print_info
 *
 * Short Description: Print an expression as a 2D expression (it's possible to
 *										switch between 2D and text view). Also let the user to move
 *										the printed expression and some key functions of the list
 *										view are available.
 *
 * Description:
 *	-
 *
 ********************************************************************************
 *********************************************************************************/

#include <tigcclib.h>
#include "fmanage.h"

#include "structures.h"
#include "global.h"
#include "calc.h"

//
void win_reopen (CONFSTRUCT *confstruct, WINDOW *ListWindow, WIN_RECT *ListWinr, int print_mode);
void print_info (CONFSTRUCT *confstruct, WINDOW *ListWindow, FORMULADATA *formuladata, const char *StrToDisplay, int print_mode, int datanr);
//


int view2d (CONFSTRUCT *confstruct, FILE **datafile, const char *StrToDisplay, WINDOW *ListWindow, WIN_RECT *ListWinr, int *datanr, int *print_mode)
{
	FORMULADATA formuladata;
	TEXTDATA textdata;

	int gk = 0;
	int x = 2;
	int y = ((WinHeight(ListWindow) / 3) * 2);
	int NumOfBlocks = 0;
	char print2Dstr[SOLVE_STRING_SIZE + 1] = {0};

	win_reopen(confstruct, ListWindow, ListWinr, *print_mode);

	//Save stack pointer
	unsigned char * temp_top_estack = top_estack;

	//Load the number of blocks
	NumOfBlocks = fmanage(datafile, NULL, 0, NULL, NULL, 0, 0, FMF_RETURN_FINFO, FMF_RETURN_NOB);

	ST_helpMsg("Working... Please wait!");

	while (1)
		{
		  TRY
		  	reset_control_flags();
		    push_END_TAG();
				WinClr(ListWindow);

				if (*print_mode != 1)
					WinStr(ListWindow, "\n\n\n\n\n");		//Just a newline that is looks better and that it doesn't goes over the title

				//If StrToDisplay is NULL, load the data with the number datanr
				if (StrToDisplay == NULL)
					{
						switch (confstruct->ActiveForm)
							{
								case FORM_FORMULA:
										fmanage(datafile, ActivePath, FMF_FORMULA_FILE_VERSION, FILE_FORMULA_EXTENSION, &formuladata, sizeof(FORMULADATA), *datanr, FMF_LOAD, 0);
										switch (confstruct->ActiveList)
											{
												case FORMULA:
														strcpy(print2Dstr, formuladata.formula);
													break;
												case DESC:
														strcpy(print2Dstr, formuladata.desc);
													break;
												default:	//NAME
														strcpy(print2Dstr, formuladata.name);
													break;
											}
									break;
								case FORM_TEXT:
										fmanage(datafile, ActivePath, FMF_TEXT_FILE_VERSION, FILE_TEXT_EXTENSION, &textdata, sizeof(TEXTDATA), *datanr, FMF_LOAD, 0);
										switch (confstruct->ActiveList)
											{
												case TEXT:
														strcpy(print2Dstr, textdata.text);
													break;
												default:	//TITLE
															strcpy(print2Dstr, textdata.title);
													break;
											}
									break;
								default:
									break;
							}
					}
				else	//else show the string in StrToDisplay
					{
							strcpy(print2Dstr, StrToDisplay);
					}

				if (strlen(print2Dstr) < 1)
					strcpy(print2Dstr, "Nothing to display!");

				TRY
					//Check if there's enough space on the expression stack
					check_estack_size(sizeof(StrToDisplay));
				ONERR
					WinStr(ListWindow, "\n\n\n\n\nMemory!");
				ENDTRY

				//Push the string on the expression stack or print it in the window
				if (*print_mode == 1)
					{
						push_parse_text(print2Dstr);
					}
				else
					{
						WinStr(ListWindow, print2Dstr);
					}


				//If print in 2D is choosen, switch between EXACT and APPROX format and print the expression
				if (*print_mode == 1)
					{
				    if (confstruct->exact_approx == 1)
					    NG_rationalESI(top_estack);
					  else
					  	NG_approxESI(top_estack);

						Parse2DExpr(top_estack, FALSE);
				    Print2DExpr(top_estack, ListWindow, x, y);
					}
		  ONERR
		  	win_reopen(confstruct, ListWindow, ListWinr, 2);
		    WinStr(ListWindow, "\n\n\n\n\nFailed to print the expression in 2D!\n\n");
		    if (print2Dstr != NULL)
					WinStr(ListWindow, print2Dstr);
				//WinStrXY(ListWindow, 2, 10, "");
		  ENDTRY

	print_info(confstruct, ListWindow, &formuladata, StrToDisplay, *print_mode, *datanr);

	//Restore stack pointer
	top_estack = temp_top_estack;

	//ST_eraseHelp();

			gk = ngetchx ();

			switch (gk)
				{
					case KEY_ESC: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							return 0;
						break;
					case KEY_DIAMOND + KEY_LEFT: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							x += 16;
					case KEY_2ND + KEY_LEFT:
							x += 4;
						break;
					case KEY_DIAMOND + KEY_RIGHT: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							x -= 16;
					case KEY_2ND + KEY_RIGHT:
							x -= 4;
						break;
					case KEY_DIAMOND + KEY_UP: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							y += 16;
					case KEY_2ND + KEY_UP:
							y += 4;
						break;
					case KEY_DIAMOND + KEY_DOWN: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							y -= 16;
					case KEY_2ND + KEY_DOWN:
							y -=4;
						break;
					case KEY_UP:
							if (StrToDisplay != NULL)
								break;	//It's not possible to switch between data when a formula has been solved
							if ((*datanr - 1) < 1)
								break;
							*datanr = *datanr - 1;
						break;
					case KEY_DOWN:
							if (StrToDisplay != NULL)
								break;	//It's not possible to switch between data when a formula has been solved
							if ((*datanr + 1) > NumOfBlocks)
								break;
							*datanr = *datanr + 1;
						break;
					case KEY_MODE: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (confstruct->exact_approx == 1)
								confstruct->exact_approx = 2;
							else
								confstruct->exact_approx = 1;
						break;
					case KEY_ENTER: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (StrToDisplay == NULL)
								copy_to_clipboard(confstruct, *datanr, NULL);
							else
								copy_to_clipboard(confstruct, 0, StrToDisplay);
						break;
					case KEY_BACKSPACE: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (StrToDisplay == NULL)
								{
									switch (confstruct->ActiveForm)
										{
											case FORM_TEXT:
													new_entry_text(confstruct, FMF_EDIT, *datanr);
												break;
											case FORM_FORMULA:
													new_entry_formula(confstruct, FMF_EDIT, *datanr);
												break;
											default:
												break;
										}
								}
							return 1;
						break;
					case KEY_ON:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							off();
						break;
					case KEY_F4:	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (*print_mode == 1)
								*print_mode = 0;
							else
								*print_mode = 1;
							return 1;
						break;
					case KEY_F2: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							if (confstruct->FontLargeSmall == 1)
								confstruct->FontLargeSmall = 2;
							else
								confstruct->FontLargeSmall = 1;
							return 1;
						break;
					case KEY_F1: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
					case KEY_F6:
					case KEY_DIAMOND + KEY_F1:
								return gk;		//Return to restart process_vars
						break;
					case KEY_F5: 	//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
							return 1;
						break;
					default:
						break;
				}

		}




	return 0;
}

//========================================================================================================

void win_reopen (CONFSTRUCT *confstruct, WINDOW *ListWindow, WIN_RECT *ListWinr, int print_mode)
{
	WinClose(ListWindow);
	clrscr();

	if (print_mode == 1)
		WinOpen(ListWindow, ListWinr, WF_SAVE_SCR);
	else
		WinOpen(ListWindow, ListWinr, WF_SAVE_SCR | WF_TTY);

  WinActivate (ListWindow);

	if (confstruct->FontLargeSmall == 2)
	  WinFont(ListWindow, F_6x8);
	else
		WinFont(ListWindow, F_4x6);

	return;
}

//========================================================================================================

void print_info (CONFSTRUCT *confstruct, WINDOW *ListWindow, FORMULADATA *formuladata, const char *StrToDisplay, int print_mode, int datanr)
{
	char str[100] = {0};
	char fstr[WinWidth(ListWindow) / 2];
	int chars = 0;


		strcat(str, "Font: ");
		if (confstruct->FontLargeSmall == 2)
			strcat(str, "BIG      | ");
		else
			strcat(str, "SMALL | ");

		strcat(str, "Mode: ");
		if (confstruct->exact_approx == 1)
			strcat(str, "EXACT   | ");
		else
			strcat(str, "APPROX | ");

		strcat(str, "Print: ");
		if (print_mode == 1)
			strcat(str, "2D  | ");
		else
			strcat(str, "txt | ");
		ST_helpMsg(str);

		WinFont(ListWindow, F_4x6);
		sprintf(str, "[%2d]", datanr);
		WinStrXY(ListWindow, 2, 2, str);

		if (confstruct->ActiveList == FORMULA)
			{
				if (StrToDisplay == NULL)
					{
						switch (confstruct->ActiveForm)
							{
								case FORM_FORMULA:
										WinStrXY(ListWindow, 20, 2, formuladata->name);
												//Look how many characters of the current string can be printed out in the window
												if ((chars_in_window_on_one_line(WinWidth(ListWindow) - 35, formuladata->desc, &chars)) == 0)
													{
														strncpy(fstr, formuladata->desc, chars);
														//Cut off
														fstr[chars] = '\0';
														//Add the "..."
														strcat(fstr, " ...");
													}
												else
													{
														strcpy(fstr, formuladata->desc);
													}
											//Print the string
											WinStrXY(ListWindow, 20, 10, fstr);
										WinLine(ListWindow, MakeWinRect(1, 20, WinWidth(ListWindow) - 2, 20));
									break;
								default:
									break;
							}
					}
			}

}

//========================================================================================================

//========================================================================================================

